## 常见问题

## 是否应该尽量使用 CI/CD 工具提供的功能？

尽量使用 CI/CD 工具提供的插件/功能，好处是不需要自己去实现，可以降低维护成本。但是相对的运维人员就需
要深入学习这个 CI/CD 工具的使用，另外还会和 CI/CD 工具绑定，会增加迁移难度。

而尽量自己用 Python 等代码去实现流水线，让 CI/CD 工具只负责调度与运行这些 Python 代码，那 CI/CD 就可
以很方便地随便换，运维人员也不需要去深入学习 CI/CD 工具的使用。缺点是可能会增加 CI/CD 代码的复杂性。

我观察到 argo/drone 的一些 examples，发现它们的特征是：

1. 所有 CI/CD 相关的逻辑，全都实现在流水线中，不需要其他构建代码
2. 每一个 step 都使用专用镜像：golang/nodejs/python
   1. 比如先使用 golang 镜像进行测试、构建，再使用 kaniko 将打包成容器镜像

那是否应该尽量使用 CI/CD 工具提供的功能呢？其实这就是有多种方法实现同一件事，该用哪种方法的问题。这
个问题非常常见。

以我目前的经验来看，需要具体问题具体分析，以 Argo Workflows 为例：

1. 流水线本身非常简单，那完全可以直接使用 argo 来实现，没必要自己再搞个 python 脚本
   1. 简单的流水线，迁移起来往往也非常简单。没必要为了可迁移性，非要用 argo 去调用 python 脚本。
2. 流水线的步骤之间包含很多逻辑判断/数据传递，那很可能是你的流水线设计有问题！
   1. **流水线的步骤之间传递的数据应该尽可能少！复杂的逻辑判断应该尽量封装在其中一个步骤中！**
   2. 这种情况下，就应该使用 python 脚本来封装复杂的逻辑，而不应该将这些逻辑暴露到 Argo Workflows
      中！
3. 我需要批量运行很多的流水线，而且它们之间还有复杂的依赖关系：那显然应该利用上 argo workflow 的高级
   特性。
   1. argo 的 dag/steps 和 workflow of workflows 这两个功能结合，可以简单地实现上述功能。
