# 区块链钱包的原理

外部账户(EOA)：现实世界中某个人拥有的账户合约账户：合约的账户

- 以太坊地址（账户 ID 以及合约 ID）: 使用 Keccak-256 哈希函数从账户公钥或合约内容派生的唯一标识符。
- 以太坊名称注册中心：功能与当前中心化的全球 DNS 系统一致，但是是去中心化的。此外这玩意儿的进展比较
  缓慢，目前应用好像还不多？

以太坊地址的编码方式：

- Inter exchange Client Address Protocol (ICAP): 一种以太坊地址编码，跟目前国际银行通用的
  International Bank Account Number (IBAN) 编码比较类似。
  - 比如 `0x001d3f1ef827552ae1114027bd3ecf1f086ba0f9` 使用 ICAP 编码得到
    `XE60 HAMI CDXS V5QX VJA7 TJW4 7Q9C HWKJ D`
- 带有大写校验和的十六进制编码 (EIP-55): 它通过把地址中的部分字符转为大写来提供一定的地址校验能力，
  同时与以太坊地址兼容。
  - `0x001d3f1ef827552ae1114027bd3ecf1f086ba0f9` 使用 EIP-55 得到：
  - `0x001d3F1ef827552Ae1114027BD3ECF1f086bA0F9`，可以看到只有部分字符变成了大写，其他与以太坊地址完
    全一致。

## 区块链钱包

我们在前面介绍了，一个以太坊账号，就是一个公私钥对，私钥就是你的账户凭证绝对不能泄漏，而账户 ID 则从
公钥派生。

那区块链钱包是一个什么概念呢？我们现实中的钱包是纸币与银行卡的容器，而区块链钱包，则是区块链账户凭证
（私钥）的一个容器，它可以保存多个账号私钥。

注意区块链只保存私钥，这是一个账户凭证，而你具体的资金是记录在区块链上，就像你储蓄卡里的现金实际上也
是保存在银行保险库里，钱包里只有一张卡而已。

使用钱包的好处之一是，你可以生成许多个密钥对，也就拥有许多个账号地址。可以在不同的交易中使用不同的账
号地址，避免了隐私的泄漏。

区块链有两种钱包格式：

- 非确定性钱包：钱包中的所有私钥都是随机生成的，密钥之间没有任何关系
  - 也被称作 JBOK 钱包（Just a Bunch Of Keys）
  - 这种钱包最大的问题是，每次生成新的账号，你都需要重新备份，避免新账号丢失。
- 分层确定性钱包（Hierarchical Deterministic Wallets）：确定性钱包的所有账号，都是使用 KDF 函数从同
  一个主密钥中派生的。
  - 其好处是你只需要在生成钱包的时候备份好主密钥（通常使用 BIP-39 助记词），随时可以通过主密钥恢复出
    完整的钱包以及其中所有的账号。
  - 最常用的密钥派生方法是树状结构

钱包相关的常用规范：

- **BIP-39**: 助记词（mnemonic codes）规范
  - 主要记录了两部分算法：一是如何生成助记词，二是如何将助记词转换为 HD 钱包种子（主密钥）
- **BIP-32**：HD 钱包规范
- **BIP-43**：多用途 HD 钱包结构
- **BIP-44**：多币种和多账户钱包

其中用到的算法也都能猜到：

- 安全随机数生成器，随机选择出需要的多个助记词
- 从字符到种子，肯定是用某种 KDF 算法如 scrypt/argon2
- 从种子到私钥，会用到 HMAC 算法
- 从有限的种子中生成出无限的密钥，肯定需要拉伸随机数的熵，这显然需要用到加密 Hash 函数比如 SHA-512.
- Chain-Code 在对上层密钥进行一次 HMAC 运算后，取前 256 位作为当前账号的密钥，后 256 位作为
  Chain-Code，用于联合密钥索引、上层私钥，共同生成下一个子密钥。
  - 好处是密钥索引跟 Chain-Code 都有泄漏的可能，但是只要「上层私钥」不泄漏，整个密钥树就不可能被推倒
    出来，你的 money 就仍然安全。
