# 区块链技术

首先是学习以太坊的使用，其次是深入理解区块链原理。

区块链的三种应用场景：

- 公有链：就是公开可用的区块链，以太坊比特币都是这种
- 私有链：企业或者个人私有的区块链，通常用于开发调试，或者其他用途
- 联盟链：是指参与节点事先确定好的区块链，只对联盟内部成员开放全部或部分功能。
  - 其特点介于公有链和私有链之间，账本半公开化、交易确认速度较快，记账成本低，数据有一定的隐私性。
  - 主要用于企业联盟间的信息公开，比如银行、证券、商业集团及上下游企业等场景，联盟链对他们的公证、结
    算清算业务和价值交换网络很有帮助。

去中心化区块链现存的一些问题：

- 它实际上并不是真正的去中心化，比如假设腾讯发行了基于区块链的代币名为「区块链 Q 币」，但实际上这个
  代币的价值仍然是完全受腾讯把控的。
- 规模较小的区块链很容易遭受「51% 攻击」，因此区块链也并不是绝对安全。只是对大的区块链发起「51% 攻
  击」的可能性比较小而已。
  - 实际上已经存在许多小的区块链遭遇「51% 攻击」的案例，比如经典以太坊就曾遭遇过 51% 攻击。
- 在云原生模式的当下，有大量的矿机实际上都跑在 aws/azure 等云服务上，如果 aws 数据中心出了问题，区块
  链很可能会受到较大影响，因此我们说它也并不是真正的「去中心化」
- 区块链中的 DAO 等自治组织实际上并不平等，每个人的投票权其实是是取决于你的资本的，在区块链中既得利
  益者天然拥有更高的权力。**DAO 与 PoS 实际上仍然是资本的民主**。
  - DAO 跟目前的投资市场及企业这样的资本社会非常契合，越早加入的人有先发优势，往往可以拥有更高的权
    利。
  - Web3 只会进一步加剧贫富差距、加剧社会分层、加剧阶级隔阂。
- 以太坊的智能合约的 Bug 是一个很大的问题，因合约 Bug 而导致的资金泄漏不是一次两次了，规模也都相当
  大。
- NFT 这样的东西，是否只是在通过制造稀缺性获利？
  - 它就像各种限量款的豪车、包包等奢侈品一样？

带着这些问题的区块链，不断的发展与遭受挫折（智能合约 Bug、51% 攻击等等），未来会如何发展呢？且让我们
拭目以待吧。

## Web3.0 为什么一定要跟钱有关？

基于区块链实现的，现在被许多人称为 Web3.0 的技术，一定需要一个奖励机制来激励各种用户参与进来，因此区
块链中的 Layer 1 Token 是不可或缺的。

因为区块链中 Token 的不可或缺性，以及其中 NFT 的不可替代性，Web3.0 自身不仅是对 Web2.0 的一次革命，
更是金融领域的一次革命。

Web3.0 的本质是由区块链支撑的价值互联网。

Web3 最大的价值是：用户创造、用户所有、用户控制、协商分配。

## 路线图

- https://github.com/OffcierCia/DeFi-Developer-Road-Map
- https://github.com/dcbuild3r/blockchain-development-guide

## 智能合约开发工具

开发环境：

- https://github.com/trufflesuite/truffle: 一个智能合约工具，提供本地完善的本地测试、模拟环境
- http://dapp.tools/: 一套以太坊开发相关的 CLI 工具

区块链客户端：

- https://github.com/ethereum/go-ethereum: geth 官方客户端

solidity 常用库：

- https://github.com/OpenZeppelin/openzeppelin-contracts
- https://github.com/NomicFoundation/hardhat

其他：

- https://github.com/ConsenSys/smart-contract-best-practices: 最佳实践

## 区块链相关工作

区块链相关初创公司的方向：

- 去中心化存储
- 跨链：区块链互联技术
- DeFi 去中心化金融
- 数据存证
- 价值转移
- 数据交换
- Web3 dAPP: 去中心化应用如 Mirror、去中心化游戏

3-5 年经验，看行情或许可以拿到 30k-60k 15-16 薪

开发方向：

- 前后端项目开发，但是需要用到一些 web3 专用的库
- 智能合约开发
- 核心协议与区块链开发
- 密码学开发
- 安全
- ...

区块链相关职位的要求汇总（一般只要求一部分，更多的是加分项）：

- 要求
  - 大部分要求 2 年或 3 年以上开发经验，部分职位要求有 1 年以上区块链开发经验
  - 掌握 Go/Rust/C/C++ 中的一到两种语言
  - 对密码学算法、安全协议(如 OAuth2)、有研究
  - 熟悉分布式共识算法，如 PoW PoS dPoS PBFT Paxos Raft
- 加分项
  - 熟悉 Linux、容器技术、Kubernetes 优先
  - 熟悉底层数据存储原理(rocksdb leveldb badgerdb)，有分布式存储(Ceph/IPFS)开发使用经验优先
  - 熟悉主流区块链（以太坊/比特币/Solana/Cosmos/Hyperledger Fabric）的原理、机制，有公链开发经验优先
  - 熟悉 Solidity 智能合约编写，有 DApp/DeFi 开发经验

## 参考文档

- https://github.com/yeasy/blockchain_guide
- https://paper.seebug.org/category/blockchain/
- [区块链被黑统计 - 慢雾科技](https://hacked.slowmist.io/statistics/?c=all&d=all)
