# S3 对象存储

AWS 上最经济实惠的存储方案。

## 存储类型（StorageClass）

> https://docs.aws.amazon.com/AmazonS3/latest/userguide/storage-class-intro.html

> https://aws.amazon.com/s3/pricing/

S3 提供如下几种存储类型：

- S3 Standard
  - 顾名思义，标准类型的存储，费用最高，适合用于存储常用数据。
- S3 Intelligent - Tiering
- (SIA) S3 Standard - Infrequent Access
  - 非频繁访问数据，存储价格更低一些，但是会收取一个数据恢复的费用。
- S3 One Zone - Infrequent Access
  - 适合存一些允许丢失的不重要的数据，它的存储费用比 SIA 更低一些，其他费用跟 SIA 没区别。
- (GIT) S3 Glacier Instant Retrieval
  - GIT 文件需要毫秒级别的恢复时间，存储费用比 GIA 更低，但是 数据恢复费用更高。
- (GFR) S3 Glacier Flexible Retrieval
  - GFR 文件也不能直接访问，需要提前 Restore，Restore 操作要花钱，用时为 1 - 5 分钟。
- (GDA) S3 Glacier Deep Archive
  - GDA 文件不能被直接访问，需要提前 Restore，这个 Restore 是需要花钱的，而且需要 12h - 48h 才能完成
    Restore 操作。

存储类型的向下转换（归档），以及类型提升：

- 文件归档
  - 可以通过 S3 的 Lifecycle 功能，来配置在文件被创建 X 天后，自动向下转换为某个存储类型。
  - 根据官方文档，这会造成一定的类型转换费用，按每 1000 次类型转换请求进行计费，单价如下(us-east-1
    virginia)
    - SIA $0.01
    - GIT $0.02
    - GFR $0.03
    - GDA $0.05
- 类型提升
  - 要批量提升 S3 文件的存储类型，需要借助 S3 Batch Operation 功能
  - S3 Batch Operation 需要提供一个文件清单，可以使用 Inventory configurations 自动生成的文件清单，
    也可以自己生成个格式符合要求的 csv
  - 对于非 GFR/GDA 文件，可以直接使用 S3 Batch Operation 批量进行 Copy 操作，批量将文件 Copy 到它原
    有的位置，设置好新的存储类型即可。
  - 对于 GFR/GDA 文件，它们都不能直接被访问，需要先 Restore 才行。因此需要使用 S3 Batch Operation 先
    批量将这些文件 Restore 回来，再批量 Copy 到原有位置，设置为新的存储类型。
  - GFR 至少要存 90 天，GDA 文件至少要存 180 天，如果提前还原，会被 AWS 立即一次性收取这个存储费用。
  - 上述类型提升操作本身也会收 Batch Operation 的费用、调用 S3 的 Restore/PUT 接口的费用

## 零、成本

> https://aws.amazon.com/s3/pricing/

根据官方文档，S3 对象存储包含这几个部分的成本：

- 存储: 一般大数据相关的 bucket，存储会是其主要成本
- 请求与数据恢复：
  - 所有对 S3 的请求都要收请求费，不过这个一般不高（除非你的客户端配置有问题）。
  - 所有对不频繁访问的存储、归档存储的请求，都会产生非常高的数据恢复费用。这种归档数据一般就不应该被
    访问了。
- 数据传输：
  - S3 传出到 internet：
    - 通常 S3 外面会挂个 CloudFront 回源，这种情况下不收回源流量费，也就是这个成本为 0
  - 从 internet 上传到 S3: 免费
  - S3 与同 region 内其他 AWS 服务（如 EC2/Lambda）之间的流量：免费
  - S3 跨区域复制成本

如果你发现你的 S3 有 usage_type 为 `DataTransfer-Out-Bytes` 的成本，那只有两个可能：跨 region 访问或
者 internet 访问导致的成本。可以通过 athena 分析 CUR 数据，进一步定位到 `description` 以及
`product_to_location` 对应的值。

## 一、私网访问

通过配置 VPC Endpoint，可以直接在 VPC 内网访问 S3，避免 VPC NAT 费用。

- Gateway endpoint:
  - https://docs.aws.amazon.com/vpc/latest/privatelink/vpce-gateway.html
  - 客户端不需要做任何改动，可直接在 VPC 内生效，但是需要修改所有路由表（修改路由表会导致正在执行的
    S3 请求失败！）。
  - 不收费
- Interface endpoint:
  - https://docs.aws.amazon.com/AmazonS3/latest/userguide/privatelink-interface-endpoints.html#types-of-vpc-endpoints-for-s3
  - 需要客户端使用指定的 endpoint api 请求 s3，并且要收数据处理费

这个基本所有需要访问 S3 的 vpc 都应该在创建的时候就配上 Gateway endpoint，主要是省钱。

## 二、Endpoint

每个 Bucket 默认提供一个 REST API endpoint: `https://<bucket-name>.s3.<region>.amazonaws.com`，比如
`https://xxx.s3.us-east-1.amazonaws.com`.

如果你需要使用 S3 托管一个站点，并通过 CloudFront 代理加速，那通常应该为 Bucket 开启静态站点访问。这
样 Bucket 还会提供一个站点 Endpoint: `http://<bucket-name>.s3-website-<region>.amazonaws.com`，比如
`http://xxx.s3-website-us-east-1.amazonaws.com`

> 注意站点 Endpoint 不支持 https，建议通过 CloudFront 为它加上 HTTPS.

站点 Endpoint 与 REST API endpoint 的区别在于：

| Key difference                                             | REST API endpoint                               | Website endpoint                                                          |
| ---------------------------------------------------------- | ----------------------------------------------- | ------------------------------------------------------------------------- |
| Access control                                             | Supports both public and private content        | Supports only publicly readable content                                   |
| Error message handling                                     | Returns an XML-formatted error response         | Returns an HTML document                                                  |
| Redirection support                                        | Not applicable                                  | Supports both object-level and bucket-level redirects                     |
| Requests supported                                         | Supports all bucket and object operations       | Supports only GET and HEAD requests on objects                            |
| Responses to GET and HEAD requests at the root of a bucket | Returns a list of the object keys in the bucket | Returns the index document that is specified in the website configuration |
| Secure Sockets Layer (SSL) support                         | Supports SSL connections                        | Does not support SSL connections                                          |

看懂了上面这个表，就能理解为什么 CloudFront 的 Origin 应该配置站点 Endpoint：我们需要的是浏览器友好
的错误信息，而不是 XML.

此外直接回源到 REST API endpoint 的请求，报错时不会触发绑定的 lambda@edge 或者 cloudfront function！
这种情况下也需要使用 website endpoint.

## 三、存储类型与生命周期

TBD

## 四、Access Point

S3 Access Point 为 Bucket 提供独立的访问域名，相比 S3 本身的访问点，它有如下特性：

- 每个 Access Point 都可以配置自己的 ACL 规则、审计日志、可访问范围（公网/VPC），互不影响
- 如果你需要使用 S3 Object Lambda 提供一个不同的对象视图，就必须为 Bucket 绑定一个 Access Point，走
  这个 Access Point 的请求将会被 Lambda 函数处理。

因此，S3 Access Point 主要是一种在共享的 S3 Bucket 上，对不同的程序或者团队，分别配置对应的权限策
略、审计日志等参数的一种方式，其次它也是 S3 Object Lambda 的必要组成部分。

## 五、S3 Object Lambda

> https://aws.amazon.com/blogs/aws/introducing-amazon-s3-object-lambda-use-your-code-to-process-data-as-it-is-being-retrieved-from-s3/

S3 Object Lambda 可以为 S3 数据提供一个额外的视图，对源文件做一些修改后再返回给发起请求的客户端。

它读写 S3 的速度非常快（20M 只需要不到 300ms）。

它会收取这几部分费用：

- 调用次数计费 - 这个通常很低
- 返回的数据量计费 - 这个通常是成本最高的一部分

但是有个缺点是，它无法设置为 Public，必须要通过 Role 授权访问。目前 CloudFront 目前并不支持通过授权
访问 S3 Object Lambda，也就是说 CDN 不能直接回源到 Object Lambda，另外我们前面也提了 Object Lambda
返回数据是要收费的。

因此在文件比较大，用量比较高时，一个比较取巧的使用方法：

- 在 CDN 与 Object Lambda 之前加一个代理服务，处理授权与其他权限、状态码相关的工作。
- 代理服务收到请求后，使用对应的 key 请求 S3 Object Lambda 的 access_point
- S3 Object Lambda 收到请求后下载并修改大文件，将修改后的文件上传到 S3 的一个临时存储文件夹内，并直
  接返回临时文件的地址
  - 临时文件夹设定生命周期策略，只保留 6h
- 代理服务收到 Object Lambda 返回的修改后文件的 S3 地址，通过 302 让用户去这个新地址对应的
  CloudFront 下载文件
- 新的下载链接的链路是：CloudFront => S3 bucket，这种回源方式是不收流量费的。
  - 为了保证文件被清楚后不会 404，可以在新链路的 CloudFront 上再绑定一个 viewer response 阶段的
    CloudFront Function，发现返回为 404，就重定向回 Object Lambda 对应的 CDN 地址。
  - 同时为了避免死循环，参数里添加一个重定向计数，超过 2 就直接返回 404

这样请求链路就变成了「CloudFront => 代理服务 => S3 Object Lambda」，而且这条链路的流量会非常小。而实
际的大文件下载是 302 重定向到了新的链路「CloudFront => S3」，这部分回源流量会比较大（每个用户下载的
文件都有被修改，都不一样），但是 CloudFront 回源到 S3 是免费的。

### 创建 S3 Object Lambda

前面提了 S3 Object Lambda 只支持通过授权访问。这里实际上涉及到了多个参与方，按调用顺序排序如下：

- 客户端的 User 或者 Role
- S3 Object Lambda Access Point
- S3 Access Point
- Lambda Function
  - S3 Object Lambda Access Point 会使用客户端 Credentials 生成一个 pre-signed URL 给到 Lambda 函
    数，这样 lambda 函数就可以直接使用这个 URL 下载 S3 文件了，不需要额外的权限。
  - 但是如果还需要往 S3 上传文件，那 Lambda Execution Role 就得有 s3:PubObject 的权限
  - Lambda 需要往 S3 Object Lambda Access Point 返回数据，Lambda Execution Role 需要有对应的权限。
- S3 Bucket

我们配置时，也需要按上述链路来一个个创建相关的资源：

- 首先得有对应的 S3 Bucket
- 然后创建 Lambda 函数，这个函数的输入输出有特定的格式，详见官方文档...
- S3 Access Point
  - 有 internet/vpc 两种类型
  - 需要指定对应的 bucket 名称
- S3 Object Lambda Access Point
  - 它的参数包含前面创建的资源：Access Point ARN、Lambda ARN

创建完成后还不够，客户端得具有调用整条链路所有资源的权限，具体配置看下一节。

### 授权调用 S3 Object Lambda

> https://docs.aws.amazon.com/AmazonS3/latest/userguide/olap-policies.html

前面介绍了 S3 Object Lambda 的整个调用链，这整条链路上基本都是在使用客户端的权限（除了中间还有个
Lambda Execution Role 可以做些额外的事之外）。因此客户端需要有访问上述链路中另外四个参与方的权限。

一个客户端的示例 Policy 如下，它分别授予了客户端访问上述另外四个参与方的权限：

```json
{
  "Version": "2012-10-17",
  "Statement": [
    {
      "Sid": "AllowStandardAccessPointAccess",
      "Action": ["s3:Get*", "s3:List*"],
      "Effect": "Allow",
      "Resource": "arn:aws:s3:us-east-1:111122223333:accesspoint/my-access-point/*",
      "Condition": {
        "ForAnyValue:StringEquals": {
          "aws:CalledVia": ["s3-object-lambda.amazonaws.com"]
        }
      }
    },
    {
      "Sid": "AllowObjectLambdaAccess",
      "Action": ["s3-object-lambda:Get*", "s3-object-lambda:List*"],
      "Effect": "Allow",
      "Resource": "arn:aws:s3-object-lambda:us-east-1:111122223333:accesspoint/my-object-lambda-ap"
    },
    {
      "Sid": "AllowLambdaInvocation",
      "Action": ["lambda:InvokeFunction"],
      "Effect": "Allow",
      "Resource": "arn:aws:lambda:us-east-1:111122223333:function:MyObjectLambdaFunction*",
      "Condition": {
        "ForAnyValue:StringEquals": {
          "aws:CalledVia": ["s3-object-lambda.amazonaws.com"]
        }
      }
    },
    {
      "Sid": "AllowReadS3Object",
      "Effect": "Allow",
      "Action": ["s3:GetObject"],
      "Resource": ["arn:aws:s3:::my-bucket/*"]
    }
  ]
}
```

给用户或程序授权上述 Policy，它就能调用 S3 Object Lambda 了。

然后还不够，前面提到了 Lambda 需要往 S3 Object Lambda Access Point 返回数据，Lambda Execution Role
需要有对应的权限。

一个示例拥有读写 S3 Object，以及返回数据给 s3-object-lambda 的 Policy 如下：

```json
{
  "Version": "2012-10-17",
  "Statement": [
    {
      "Sid": "AllowObjectLambdaAccess",
      "Action": ["s3-object-lambda:WriteGetObjectResponse"],
      "Effect": "Allow",
      "Resource": "*"
    },
    {
      "Sid": "AllowReadPutS3Object",
      "Effect": "Allow",
      "Action": ["s3:GetObject", "s3:PutObject"],
      "Resource": ["arn:aws:s3:::my-bucket/*"]
    }
  ]
}
```

把如上 Policy 配置到 Lambda Execution Role 上，应该就没啥问题了。

更进一步，可以在 Access Point 上添加 Policy，限制它的用途，提升安全性。比如跟上面一致的：

```json
{
  "Version": "2012-10-17",
  "Statement": [
    {
      "Sid": "AllowReadPutS3Object",
      "Effect": "Allow",
      "Action": ["s3:GetObject", "s3:PutObject"],
      "Resource": ["arn:aws:s3:::my-bucket/*"]
    }
  ]
}
```

### VPC 中的 S3 Object Lambda

为了避免 NAT 费用，S3 Object Lambda 对应的 Access Point 应该使用 VPC 模式，这个设置起来很简单，略过
不提。

### 客户端访问 S3 Object Lambda

客户端访问 S3 Object Lambda 与直接访问 S3 bucket 唯一的区别，就是将 bucket 名称设置为 Object Lambda
的 Access Point 地址。

Python 示例如下：

```python
import boto3

s3 = boto3.client('s3')

print('Original object from the S3 bucket:')
original = s3.get_object(
  Bucket='danilop-data',
  Key='s3.txt')
print(original['Body'].read().decode('utf-8'))

print('Object processed by S3 Object Lambda:')
transformed = s3.get_object(
  # 这里是 s3-object-lambda 的 access point 地址
  Bucket='arn:aws:s3-object-lambda:us-east-1:123412341234:accesspoint/myolap',
  # key 可以带有 HTTP params，这部分信息可以被 s3-object-lambda 函数读取到
  Key='s3.txt?xx=aa&yy=bb')
print(transformed['Body'].read().decode('utf-8'))
```

如果是公网的 Object Lambda Access Point，还可以直接在 S3 Console 页面上查看使用，跟直接看 S3 Bucket
区别不大。

但是如果是私网的 Object Lambda Access Point，直接在页面上查看就会报错，只能在 VPC 内通过 SDK 访问。

## 四、S3 指标分析

[S3 Storage Lens](https://s3.console.aws.amazon.com/s3/lens) 提供对 S3 bucket 强大分析能力，列举如
下：

- 账号级别、Region 级别、Bucket 级别的指标分析能力
- 免费指标
  - 存储桶级别汇总的使用情况指标，只保留 14 天
- 收费指标（$0.20 per million objects monitored per month）
  - **活动指标**：请求数、上传/下载字节数、按存储桶汇总的错误数
  - **CloudWatch 指标**：在 CloudWatch 中访问存储桶级别、账户级别和组织级别的使用情况和活动指标
  - 按**前缀聚合**：按 prefix 汇总指标，可以通过调整「前缀存储阈值」、「前缀深度」来减少指标数据

按前缀分析 S3 使用情况变化，可以定位到导致成本上涨的文件路径，从而做针对性的优化。这对共享 S3 Bucket
的成本分析与控制非常有用。

## 五、S3 日志分析

首先在 s3 bucket 配置中打开请求日志，然后使用 athena 建表分析，详见官方文档
[using-s3-access-logs-to-identify-requests](https://docs.aws.amazon.com/AmazonS3/latest/userguide/using-s3-access-logs-to-identify-requests.html)

但是存在一个问题是，s3 的日志拆得太细了，每秒钟都可能会上传 2 到 8 条日志记录，导致使用 athena 查询
非常慢，而且很容易触发 S3 API 速率限制。

解决方法见此文件夹的 Athena 部分
