# AWS 成本分析

## 一、成本管控方案

我目前的经验：

- 设计一套 Tag 规范+Tag 管理服务，确保绝大部分的资源都有对应的负责人，并且能灵活变更（比如批量修
  改）。
  - 关注 Tag 不规范（或无 Tag）的资源成本，使用各种手段，将它维持在较低的比例。
  - 对于跨域流量、NAT 流量等无法通过 Tag 在业务上进行区分的成本，也要保持关注。
- 统一的资源创建入口，必须选择或填写好规范的 Tag，然后通过资源创建服务执行创建。
  - 可以嵌入到内部工单服务里，让业务侧按格式填写资源创建工单，SRE 审核后再自动创建
  - 也可以考虑用 gitops 的方式，比如 terraform/pulumi，走 git review 的流程
- 设计一套成本服务，从 AWS CUR 查询数据，然后基于 Tag 的各个维度进行成本分析，将分析完成的数据上传到
  S3 或者直接放数据库里
- 然后通过 Grafana 等报表工具为不同的业务团队制作交互式报表
  - 以及为 SRE 分析细致数据，制作更底层的交互式报表，降低成本分析难度。（手写 SQL 还是比较繁琐的，特
    别是过滤条件多的情况下）

## 二、CUR 账单

- [Setting up an Amazon S3 bucket for Cost and Usage Reports](https://docs.aws.amazon.com/cur/latest/userguide/cur-s3.html)

对于 AWS Organizations 而言，CUR 是一个整合账单，因此提供了 `blended_cost` 这个字段来记录组织的混合
成本。

AWS 将组织中的所有成本汇总到管理账户进行成本计算并应用上合同的优惠费率，然后根据使用比例再分配到每个
成员账户，直接查询成本账户账单中的 `blended_cost` 即可得到如此处理之后的混合成本。

CUR 账单中存在部分不带标签的资源成本，在进行成本分析时，可通过 `line_item_resource_id` 或者其他字段
识别到它们，特殊处理为它们补上标签。这类资源成本包括：

- CloudFront
  - CloudFront Lambda@Edge
  - CloudFront Function
- S3
  - Object Lambda: 这个的成本完全查不到...

CUR 账单的 usage_type 字段详细说明：

- [Understanding your AWS billing and usage reports for Amazon S3](https://docs.aws.amazon.com/AmazonS3/latest/userguide/aws-usage-report-understand.html)
- [Interpreting your AWS bill and the AWS usage report for CloudFront](https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/billing-and-usage-interpreting.html)

## 三、AWS 计算费用折扣

针对按需使用的计算实例（OD），AWS 提供了 SP 与 RI 两种标准的折扣方式。

> 当然量大了之后，CDN、Spot 实例等资源的成本也都是可以谈折扣的，但是不在这里的讨论范围内了。

### 1. Saving Plans

SP 的逻辑是，被 committed 的费用 cover 的 OD 实例，会使用一个优惠后的价格计费。针对不同的 SP 合同，
每种实例的优惠力度也会有变化，具体的优惠比例及价格参见
[Compute Savings Plans - Pricing](https://aws.amazon.com/savingsplans/compute-pricing/).

SP 有这几种类型：

- Compute: 更灵活，没有实例、区域限制，推荐选这个。
- EC2 Instances: 比 Compute 大概要多 3% 的折扣，但是限制实例类型、区域，灵活性会差些。
- SageMaker: 这是 SageMaker 专用的成本节省计划
- AmazonRekognition: 这个单纯按 API 调用次数收费，不需要创建什么实例，标签也就无从谈起

举例来说，对于一年期、不预付费的 SP 计划，被 SP cover 的实例成本总体上能优惠 27%. 如果调整成三年期、
不预付费的 SP，大概能优惠 50%.

全部预付费、完全不预付费，价格大概相差 6%.

注意点：

- 多个 SP 计划可以叠加，而且一个大的 SP 计划跟多个小的 SP 计划，总体折扣没什么区别。
- 可以按需确定，比如混合购买一年与三年期的 SP 计划
- 全部预付与部分预付，差 2 个点左右的折扣
- 一般建议先 cover 住 80% 看看效果，看情况再决定是否往上加 SP
  - 还要考虑到 SP 计划内可能出现的资源变动，避免 SP 承诺过于激进导致成本浪费
- SP 跟 RI 都可能会影响到 AWS CUR 数据内容，导致成本报表的数据失准。
  - 可以直接直接使用 `pricing_public_on_demand_cost` 为权重进行二次成本拆分来解决这个问题。

### 2. Reserved Instances

RI 跟 SP 的 `EC2 Instances` 类型一样，都限制区域与实例类型，不建议直接使用 RI 去覆盖 EC2 计算成本。

RI 的优势在于，它能覆盖托管 EC2 实例的成本，也就是这几项：

- RDS
  - 如 Aurora MySQL、Aurora PostgreSQL
- ElastiCache
  - 主要是 Redis
- OpenSearch/ElasticSearch

在 RI 的「Savings opportunities」会列出来上面这几项的潜在成本优化空间。

可以考虑单独为上述这几种托管 EC2 的实例成本购买 RI，但是要注意 RI 的实例类型跟区域是固定的，买了就改
不了了，所以谨慎些，不建议买太长。

## 四、AWS 成本管理面板

AWS 官方有提供一个成本管理面板，下面简单介绍下。

### 1. 成本报告

可以从各种维度创建成本报告。

但是基于 CUR 数据与 Grafana 自己做报表会细致许多，应该大部分公司也都是这么干的，所以这个功能用得人比
较少。

### 2. 预算管理

设定一个预算，当超过或者快接近预算时，就触发告警。

预算管理好像说是有 62 个预算天的免费额度，超过这个额度就需要付费。

这个我从没用过。

### 3. 成本异常检测 Cost Anomaly Detection

使用机器学习算法，定期检测成本突增之类的成本异常问题，如果检测到的成本异常超过设置的一个阈值（比如
$100），就会触发告警。

这个感觉还有些用，可以配几个告警，每天值班人看一下。

免费功能，但是如果使用了短信告警，会额外收取短信或邮件的费用（频率不高的话，这个成本基本可忽略）。

## 参考

- Improving cost visibility and allocation - AWS Re:Invent 2021
