# 容器安全

## 加强 runc 容器的安全性

使用 runc 运行的容器，可以通过如下几种方式加强安全性：

1. 非特权容器
1. 非 root 账户 - 可能遇到文件夹映射没有权限的问题
1. 根文件系统设为只读，动态数据写入数据卷中

## 安全容器技术

Linux 容器相比传统虚拟机，最大的优势是「敏捷」和「高性能」但是也存在明显的问题——隔离得不彻底，共享了
系统内核，存在很多安全隐患，容器越狱的难度要比虚拟机低非常多。

以 cgroup/namespace 为核心的容器技术，容器与宿主共享 Linux 内核，一共有7个攻击面（来自美团的容器安全
文章）：Linux Kernel、Namespace/Cgroups/Aufs、Seccomp-bpf、Libs、Language VM、User Code、Container
engine

总之它很简单有效，但是不够安全，因此衍生出了一些加强安全性的容器技术：

1. [kata containers](https://github.com/kata-containers/kata-containers): kata 容器本质是轻量级虚拟
   机，但是性能和使用上和传统容器类似。
   - kata 可以直接通过 cri-o/containerd 接入到 k8s
     中：[how-to/run-kata-with-k8s](https://github.com/kata-containers/documentation/blob/master/how-to/run-kata-with-k8s.md)
1. [google/gvisor](https://github.com/google/gvisor): google 开源的安全容器技术，其本身是一个用户态
   Linux 内核，借此实现容器的内核隔离。
   - gvisor 也可以通过 containerd 接入 kubernetes:
     [gvisor with kubernetes](https://github.com/google/gvisor/blob/master/g3doc/user_guide/quick_start/kubernetes.md)
1. [firecracker-microvm](https://github.com/firecracker-microvm/firecracker): aws 开源的微虚拟机技
   术，使用 rust 实现，细节待研究。

更多的容器运行时可以在
[Container Runtime - CNCF Landscape](https://landscape.cncf.io/category=container-runtime&format=card-mode&grouping=category)
中找到。

上述这些安全容器技术全都兼容 OCI 规范，可以和 kubernetes 无缝集成。

目前美团阿里等国内大厂正在转向 kata containers，比如阿里云的 kubernetes 容器运行时就能选择「安全沙
箱」。而 gvisor 被 google 应用在自家的产品中。

> 安全容器的主要应用场景是多租户，比如 serverless 的公有云环境（阿里云 serverless kubernetes），各用
> 户之间和完全不能互相信任的。这种场景下就必须要加强各租户之间的隔离性，runc 容器的隔离性就不够用
> 了。

## 相关文章

- [云原生之容器安全实践-美团技术团队](https://tech.meituan.com/2020/03/12/cloud-native-security.html)
- [基于Rust-vmm实现Kubernetes运行时](https://zhuanlan.zhihu.com/p/188500726)
