# 容器镜像相关的问题

## 同一份源码，多次打 Docker 镜像结果不一致

因为各种各样的原因，对同一份源码多次打容器镜像，结果可能是不一致的。原因有：

1. 底层镜像更新了，比如 debian:buster 就永远是最新的小版本。
2. 构建环境变更了。比如多阶段构建的 sdk 镜像更新了，或者依赖库更新了。
3. 即使镜像构建环境完全一致，因为时间戳等原因，也会导致镜像 Hash 摘要不一样。

总之就是即使源码没有变化，多次构建出的镜像也不会一致。

如果要求比较严格，那生产环境只应该上线被测试过的镜像，也就是要求「镜像一致」！可开发人员通常都以 Git
仓库为中心，大家更希望只要求「源码一致」。

如果只要求「源码一致」，整个 CI/CD 流程可以简化很多，开发测试环境和生产环境可以完全隔离，只需要交付
源码。

而如果要求镜像一致，那测试通过的镜像必须被传递到生产环境，必须交付测试环境的镜像到生产环境，流程会更
复杂。

这里的分歧，源自「容器镜像构建不可重复」这个问题。

## 涉及到「源码交付」或者「重复构建容器镜像」的工具

Buildkit、Docker-DIND

## 镜像的分层分析工具

- [dive](https://github.com/wagoodman/dive): 可以分层浏览镜像中的所有文件，主要用于分析镜像每一层中
  潜在的无用文件，找到优化镜像大小的方法。

## Dockerfile 中 chown 导致镜像膨胀的问题

> 相关 Issue: <https://github.com/moby/moby/issues/6119#issuecomment-268870519>

1. 跟存储驱动有关，有的驱动如 `devicemapper` 的 `chown` 不会造成镜像膨胀，但是 `aufs` 会有这个问题。
