# 存储器种类、接口与协议

- SD/microSD 卡: SD 即 Secure Digital 的缩写，意即安全数字卡，是一种安全闪存存储卡，由美国的非盈利组
  织 SD 协会 (SDA) 制定标准。
  - 8 到 9 个引脚，最便宜，速度最慢，主要用于制作可插拔的超小存储设备。
  - SD 卡的接口与协议是绑定关系。
  - 针对嵌入式设备，SD 卡支持通过 SPI 协议进行读写。
  - 此外还有 1bit / 4bits 三种 MMC 总线模式，分别支持不同速率的读写。
  - UHS-II 以上的卡需要额外的拓展引脚，目前市面上 UHS-I 及以下的卡是绝对的主流。
- eMMC(embedded MMC): 速度比 TF 卡快一点，也贵一点。主要用在便宜的计算机/手机设备中，一般是焊在 PCB
  上不可拆卸。
  - 其协议与 SD 卡高度兼容，所以甚至也有很多转接板能直接将 eMMC 转成 SD 卡接口，再插入使用。
  - 因为高度兼容，所以 MCU 驱动一般也直接就叫 SDMMC，一起给实现了。
  - 与 SD 卡的协议对比，它不支持 SPI 模式，但是对 MMC 1bit / 4bits 均有支持，另外还额外支持 8bits /
    DDR 模式。
  - 既然协议高度兼容，显然 eMMC 跟 SD 卡一样，也只支持半双工运行，即读写必须分开执行。eMMC 5.0 规范
    理论速度也仅 280M/s，只到 SATA2.0 的速度。
- UFS: eMMC 的替代品，其物理结构是由多个闪存芯片、主控、缓存组成的阵列式存储模块，支持全双工运行，可
  同时执行读写操作，并且支持指令队列。
  - 更快，当然也就更贵，主要用在中高端安卓手机上。同样一般也是焊在 PCB 上不可拆卸，不过 2016 年三星
    开始推出基于 UFS 的可插拔存储设备。
  - 读取速度基本都 700MB/s 以上，越新的规范速度越快。
  - UFS 不知道啥接口，反正暂时玩不到也就不需要了解。
- 固态硬盘 SSD: 固态硬盘跟上面几种一样也是用闪存存储数据，但是它尺寸要大很多，散热、布局等方面都更有
  余裕。可通过在 PCB 上使用多颗存储芯片、多通道并发读写提升速度，速度就快很多。
  - 高速 SSD 都是使用 M.2 接口 + NVMe 协议。NVMe 也是当下速度最高的存储读写协议，将速度直接提升到
    3200M/s，甚至现在的 PCIe4.0 最低都 7000M/s 了。
  - [iPhone 6s](https://www.zhihu.com/question/36051124) 就开始使用 NVMe 协议的 SSD 作为其存储，比安
    卓的 UFS 还是强很多的。
- 机械硬盘 HDD: 机械硬盘速度一般般，主要用 SATA3.0 接口与协议，理论速度一般是 6Gbits/s。

其他补充：

- SDIO 卡：是 SD 标准的一个拓展以支持 IO 相关功能，主要应用场景是在嵌入式领域通过 SD 接口接入 WiFi/
  蓝牙模块、以太网模块、GPS 模块等等外设。
  - SDIO 支持 SPI / 1bits 模式，其全速规范还额外支持 4bits 模式

## SD 卡规范

以接口版本划分，SD卡可分为普通速度、高速度以及UHS-I、UHS-II。接口版本制约了该存储卡的最大存储带宽，
但并不代表存储卡能实际达到这样高的速度。UHS-I标准规定了最低50MB/s，可选104MB/s的最大带宽，换句话说使
用UHS-I接口的SD卡最高读取不会超过104MB/s。UHS只是接口版本规范，决定了最大带宽，但并非实际工作速度的
标称。

UHS-II增加了全新的针脚定义，接口带宽最高可达到312MB/s，UHS-II存储卡向下兼容UHS-I设备，不过在UHS-I设
备上使用时会被限制到UHS-I的最大带宽。

以写入速度划分，SD卡有Class等级与UHS速度等级两种速度标识方法。Class等级用于普通速度和HS高速类型接口
的SD卡。UHS则有两个速度等级：UHS Speed Class 1（U1）和UHS Speed Class 3（U3），分别对应最低持续写入
10MB/s和30MB/s以上。

在Class速度等级与UHS速度等级之外，SD卡协会今年又提出了一个新的视频速度等级，覆盖了现有的Class
6、U1（Class 10）、U3三种规格，又额外增加了60MB/s写入速度的V60以及90MB/s的V90规格，其中V90是专为8K视
频录制而设。

虽然新速度标识的出现是为了给消费者提供选购参考，但在新的速度标识出现的同时，旧的划分标准并未取消，这
使得SD卡上的标识可能会越来越多，使得一些不熟悉的人产生迷惑。

## 市面上的 SD 读卡器与读写速度

> 以感觉国内最烂大街的川宇读卡器为例

首先最便宜的那种，肯定不支持 UHS-I，插再牛逼的卡，速度也破不了 10M/s（比如川宇最便宜的 9.9 二合一读
卡器）.

稍微贵一点的，30 块以内的读卡器，基本都只支持到 UHS-I，写入能到 50M/s 就谢天谢地了（比如川宇卖最火爆
的 USB3.0 二合一读卡器，不到 20 块钱），到这个级别，读写时 SD 卡就已经比较烫手了。

UHS-II 需要添加拓展引脚，市面上非常少见，这也导致对应的读卡器贵很多。支持 UHS-II 的读卡器，比如川宇
51 块钱的 UHS-II 读卡器，最高就能跑到 260M/s 的读取速度了，写也有 TF 卡 100M/s、SD 卡 200M/S，不过到
这个级别估计卡片就更烫手了。（但是目前貌似主流的 TF 卡都仅支持到 UHS-I，而且前提是使用 9 针的卡套，
纯 8 针的 TF 卡，写速度还要砍半）

川宇更牛逼的是 89 块钱的 CFast2.0 读卡器，能跑到 500M/s，仅支持 CFast2.0 协议。

川宇最贵的 289 的读卡器，仅支持 CFexpress 卡，已经需要铝合金散热槽了。官放实测写能到 750M/s，读能到
880M/s。
