# FileSystem - 文件系统

文件系统，也就是将物理磁盘抽象为一个文件树，提供给用户使用的一个磁盘管理系统。

目前主流的三大操作系统，主流的文件系统都不同。

1. Linux: 主流是 ext4/xfs/zfs/btrfs
   1. 内存存储使用 tmpfs
1. Windows: 主要是 NTFS。以前是 FAT32，单个文件最大能到 4G。
1. MacOS: HFS 和新出的 AFS

## Union File System / Disk Arrays

一块磁盘的容量总是有限的，而且也有单点问题——万一磁盘坏了，数据就完蛋了。

为了获得更大的存储空间，或者实现磁盘的高可用，就需要用到磁盘阵列（Disk Arrays）。

磁盘阵列是一种将多个磁盘合并成一个文件系统来使用的技术，最常用于 NAS(Network Attached Storage) 技
术。通过数据的分割写入并发读取等技术，磁盘阵列能获得比单块磁盘更快的读写速度。通过增设冗余，磁盘阵列
可以容忍一定的磁盘损坏。

代表性的技术有：

1. RAID: 通过硬件/软件
   - RAID 是直接接管所有磁盘设备，使用硬件阵列卡时效率很高，机房标配。
   - RAID 有多种冗余/提升性能的策略，目前主要使用的有如下几类：
     1. RAID0：多盘并行读写加速，没有任何冗余。坏掉任何一块盘都会导致阵列数据无法恢复。
     1. RAID1：为每个磁盘设置一个镜像盘，数据读写同时在这两个盘上进行。
     1. RAID10：并行读写+磁盘镜像，需要至少 4 块磁盘。性能高数据安全性高，但是有一半的空间都被用做了
        冗余。
     1. RAID5: 各盘互相保存冗余信息，算法复杂性能相对低，但是空间利用率高。总共只设了一块盘大小的冗
        余，因此最多只能坏一块盘。
   - 相同容量的磁盘往往寿命长度也差不多，因此用新盘组 RAID 容易导致同时坏多个盘，数据无法恢复！！！
     尤其是 RAID5，风险很高。
   - RAID5 坏一块盘后，修复速度很慢！
   - RAID 10 可以直接热插拔**一块**硬盘，重新插入硬盘时，会自动恢复数据。就是说坏一块盘的情况下可以
     直接更换新盘，不影响系统。
2. mergefs：mergefs 将多个**已挂载的文件系统**合并（虚拟）成一个新的挂载点。
   - 它不直接管理磁盘设备，各个硬盘仍然是相互独立的。因此坏掉一个硬盘，只会丢失那一部分数据，不会导
     致所有数据彻底丢失。

### 参考文档

- [raid5 磁盘阵列真的不安全么？ - 知乎](https://www.zhihu.com/question/20164654/answer/348274179)

## Distributed File System

磁盘阵列是在一台主机上插多个磁盘，通过磁盘阵列卡或者软件的方式将多个磁盘合并成一个存储空间。

但是规模再增大，我们会发现一台主机的存储空间仍然不够，性能可能也无法满足要求。另外我们发现主机或磁盘
阵列卡本身，也存在单点问题。

为了解决这个问题，就需要将数据存储到多台主机，多个磁盘阵列中。使用网络连接这些主机，得到的文件系统被
称作「分布式文件存储系统」。

代表性的开源技术：

1. Ceph: 从网上资料来看， ceph 应该是应用最广泛的开源分布式存储
2. GlusterFS: 也经常看到各类相关的对比文档，但是流行度不如 Ceph
3. HDFS: 专为大文件存储设计的分布式文件系统，主要被应用在大数据领域。
4. MinIO: 目前最流行的云原生对象存储方案，性能高，部署维护方便。

这类技术适合集群存储，机器与存储量小的情况下搭建分布式存储是浪费资源。

除了 Ceph/GlusterFS 外，还有一类专门为大数据分析设计的分布式存储：HDFS

## 文件传输协议

有了上述文件系统后，远程的应用如果想要方便地使用这些存储资源，就需要用到文件传输协议。

日常接触得最多的，肯定是 HTTP/FTP 协议。但是 HTTP 只是能提供单纯的文件传输功能，FTP 也只提供有限的文
件查询功能。

我们希望的，是像使用本地的存储空间一样地去使用远程存储资源。这种技术叫网络文件系统。

使用的最多的网络文件协议有：

1. NFS 协议：Linux 世界（服务器）使用广泛，比如 Kubernetes 就支持挂载 NFS 存储。
1. SMB 协议：Windows 世界使用广泛，各种 Windows 共享文件夹，都是使用的这个协议。
   - linux发行版/macos/windows 的文件管理器，都支持直接访问 `smb://` 存储。
1. ISCSI: iscsi 比 NFS/SMB 更底层，它直接将裸磁盘提供给客户端。
   - 简单解释：NFS/SMB 是共享文件夹，而底层的文件系统由服务端管控，对客户端是透明的。而 iscsi 是直接
     共享存储块(底层是物理磁盘)，客户端直接读写底层裸磁盘，因此文件系统(NTFS/EXT4 等)完全由客户端控
     制。

## 参考

- [块存储、文件存储、对象存储这三者的本质差别是什么？ - 知乎](https://www.zhihu.com/question/21536660)
