# [Module（模块）](https://docs.ansible.com/ansible/latest/modules/modules_by_category.html)

Ansible Module 就是能够用于执行某项任务的一个组件，Ansible 维护了一个庞大的 Modules 库供用户使用。
Module 是 Playbook 的组成部分，每一个 Playbook 的步骤，都对应一个 Module.

```
# 列出所有的 modules
ansible-doc -l
# 统计模块数量
ansible-doc -l | wc -l  # 目前有 3387 个
# 搜索模块
ansible-doc -l | grep <key-word>
```

## 常用模块

### 1. [远程执行命令](https://docs.ansible.com/ansible/latest/modules/list_of_commands_modules.html)

1. `command`: ansible 直接调用对应的程序。不使用 shell，因此用不了 shell 相关的语法（比如 shell 内建
   命令 `cd`、管道符 `|`、字符串插值）。
   1. 这对应 python 的 `subprocess.run(shlex.split("<cmd>"), shell=False)`
2. `shell`: 通过远程的 shell(`/bin/sh`) 来执行命令，可以使用 shell 语法。
   1. 对应 python 的 `subprocess.run("<cmd>", shell=True)`
3. `script`: 将本地的脚本传输到远程主机上执行，可指定脚本执行程序(`/usr/bin/bash`
   `/usr/bin/python3`).
   1. 不要求远程主机上安装有 `python`
4. `raw`: 直接通过 ssh 在远程主机上执行命令.
   1. 不要求远程主机上安装有 `python`
   2. 官方文档说它和 `shell` 的区别在于它在实现上更 `dirty`。因此还是优先选用 `shell` 模块

### 2. [文件处理](https://docs.ansible.com/ansible/latest/modules/list_of_files_modules.html)

1. `copy`: 将文件拷贝到远程主机上
2. `fetch`: 从远程主机上抓取文件到本机
3. `file`: 文件(夹)的创建、删除、权限修改等。
4. `archive`/`unarchive`: 压缩/解压
5. `synchronize`: 数据同步，对 `rsync` 的封装。
6. `find`/`replace`: 对应常用的 `find`/`sed` 两个工具。
7. `get_url`: 通过 `http/https/ftp` 协议下载文件。
   1. 在下载大文件时，在本机下载好然后 `copy` 到远程主机上可能是更好的选择。

### 3. Linux 系统运维

1. 定时任务管理：`cron`
2. 用户/群组管理: `user`/`group`
3. 包管理: `apt`/`yum`/`apk`/`pip`
4. 主机信息（IP 地址、OS 信息等）: `setup`

### 4. 「云」上运维：Terraform

Terraform 是一个倡导「基础设施即代码」的基础设施管理工具，被广泛应用在云上资源各种资源的 CURD，比如
云主机的创建、属性变更及销毁。由于其在资源生产成功之后会在本地以一个 state 文件的形式记录整个资源的
详细信息，而这些信息的记录使得整个模板所定义的资源可以保证前后端的高度一致性，可以有利于后续对于整个
一套资源的有效的版本控制。同时 Terraform 还提供了 DataSource 可以用于查询已有资源的各项属性(IP 地
址、规格、可用区、使用的镜像、所属的虚拟网络等等)。

而 Ansible 主要被用于远程主机的环境配置（软件安装、配置）、状态控制（主机的开/关/重启），虽然
Ansible 也能实现部分 terraform 的功能，但是 Ansible 没有 state 文件，也没有 datasource，所以基础设施
还是建议用 terraform 管理。

为了更好地结合两者，可以通过 ansible 的 terraform 插件调用 terraform:

1. [`terraform`](https://docs.ansible.com/ansible/latest/modules/terraform_module.html): 通过
   `ansible` 调用 `terraform`
