## 基础设施的管理与维护

## 基础设施的搭建与初始化

使用 pulumi/terraform/ansible 进行基础设施的搭建与初始化，尽可能的自动化整个流程。

编写与使用这些自动化代码或者配置时，就要考虑好它们的管理和维护。

## 基础设施的自动化测试

对于 pulumi 代码，可以使用 pytest 等单元测试框架来进行测试。

系统搭建完成后，可以设计一套系统的自动化测试工具，来对整个系统进行一个自动化的测试。

## 备份与恢复

对于存有重要数据的系统/虚拟机/磁盘，一定要设置好自动备份机制，而且最好是有多地备份。

备份有了，还需要考虑如何能够快速恢复，要有完善的自动化恢复功能，或者退而求其次——有清晰的文档记录好了
备份的恢复流程。

对于 Etcd/Redis 这类内存数据库，可以使用 ansible 等工具定期进行数据持久化，然后再备份。另外对一些有
对应的备份命令或 API 的软件，也可以通过这些命令/API 来备份数据。而对于其他文件形式的数据，则可以考虑
直接使用 rsync 等数据同步工具进行数据备份。

## 定期进行系统维护 - Regular System Maintenance

不论是单台 Linux 虚拟机，还是一个 Kubernetes 集群，或者更复杂的系统，都需要定期进行系统维护。

对于 Linux 系统、或者更上层的应用软件而言，**更新系统组件——尤其是安全补丁(security patches)——是一件
非常重要的任务**。

但是任何更新，都存在一些风险，或者过于繁琐。所以很多「保守派」系统管理员，为了规避风险，避免麻烦，一
旦系统稳定了，他们就从来不进行任何更新——不论是 OS 更新还是上层应用更新。

对于 Kubernetes 集群的所有节点，我们可以分批地，使用 ansible 来进行自动化的系统更新与系统维护。其实
最好是也能有类似 flagger 灰度发布一样的功能，能够实现渐进式地、自动地更新所有 Kubernetes 节点。

更好的是，我们有维护一套进行系统验证的测试程序，能用于确保系统更新后其状态仍符合预期。而上述的这个灰
度更新工具，也能使用这套测试程序来验证系统状态。

## 混沌工程

除了定期的系统维护，操作系统在长时间的运行、维护和使用过程中，也会渐渐差异化。

这导致的几个典型问题有：

- 虚拟机重启后，某些功能出现问题。比如 Service 没自动启动、磁盘没自动挂载。
  - 这通常是系统管理员自己疏忽了，没配置自动启动/磁盘自动挂载，但是没重启过，很难发现问题。
- Kubernetes 集群：同一个 Pod，在别的节点可以跑，在这个节点就跑不起来。

因此我们应该引入混沌工程，随机地杀死、删除 Pod，重启、删除集群节点等等，以确认集群的健壮性，发现问题
就及时修复，避免造成更大的损失。

## 参考

- [How Often Should I Update our Linux Server?](https://serverfault.com/questions/9490/how-often-should-i-update-our-linux-server)
- [How often should I reboot Linux servers?](https://serverfault.com/questions/15782/how-often-should-i-reboot-linux-servers)
- [Docker 方式部署的应用的版本更新](https://www.cnblogs.com/kirito-c/p/11338398.html)
