# 基于拓扑的路由或转发策略

请求优先路由到同一个可用区的节点，这可以节约大量跨 Zone 甚至跨 Region 的流量，同时降低延迟。

但是也有潜在风险：同可用区的实例负载很高，而扩容到其他可用区的实例却几乎没负载。形成传说中的「一核有
难，七核围观」现象。

## Kubernetes Service - [Topology Aware Hints](https://kubernetes.io/docs/concepts/services-networking/topology-aware-hints/)

> Kubernetes 1.21+ 添加的 Alpha 功能，并且预计在 1.23 升级为 Beta 特性，默认开启。

待续

## Istio - locality-prioritised load balancing

官方文档着重介绍了多集群模式下的本地负载均衡策略，但是这个功能应该也适用于单集群、多可用区场景下，基
于 Zone 的本地负载均衡才对。

官方文档语焉不详，不过猜测只需要配置 `spec.trafficPolicy.localityLoadBalancerSetting` 和
`spec.trafficPolicy.outlierDetection`，就能启用基于 Zone 的本地负载均衡策略，具体的待研究：

- [LocalityLoadBalancerSetting - Istio Docs](https://istio.io/latest/docs/reference/config/networking/destination-rule/#LocalityLoadBalancerSetting)
- [OutlierDetection - Istio Docs](https://istio.io/latest/docs/reference/config/networking/destination-rule/#OutlierDetection)

一个示例配置：

```yaml
apiVersion: networking.istio.io/v1beta1
kind: DestinationRule
metadata:
  name: helloworld
spec:
  host: helloworld.sample.svc.cluster.local
  trafficPolicy:
    connectionPool:
      http:
        maxRequestsPerConnection: 1
    loadBalancer:
      simple: ROUND_ROBIN
      localityLbSetting:
        enabled: true
        failoverPriority:
          # 这个列表表示的优先级顺序如下：
          ## 1. 首先查找两个标签都匹配的 endpoints，它的优先级最高
          ## 2. 其次，查询 region 标签相同的 endpoints
          ## 3. 最后，其他所有 endpoints 的优先级都相同，全都是最低优先级
          - "topology.kubernetes.io/region"
          - "topology.kubernetes.io/zone"
    outlierDetection:
      # 如下参数仅适合测试！
      consecutive5xxErrors: 1
      interval: 1s
      baseEjectionTime: 1m
```
