# 常见问题排查指南

## CPU 突增至 100%

1. 内存用尽

## 磁盘空间告急

在 Linux 或者 Unix 系统中，直接手动删除文件将会从文件系统的目录结构上解除该文件的链接（unlink），然
而如果文件被删除时仍然有一个进程正在使用它，那么该进程仍然可以读取此文件，磁盘空间也会一直被占用。因
此如果因为日志等文件不断增长导致空间告急，直接删除这些日志文件是没有用的！

正确的操作是使用 echo 等命令清空文件：

```
# 直接清空日志文件（文件将仅包含一个换行符）
echo "" > /path/xxx.log

# 只保留最新的 10000 行日志
tail -n 10000 /path/xxx.log > /path/xxx_tail.log
cat /path/xxx_tail.log > /path/xxx.log
rm /path/xxx_tail.log
```

### 手动删除文件后的补救办法

手动删除文件后，通过 `df -h` 会发现磁盘空间没有变化，但是使用 `sudo du -sh /*` 却可以看到文件夹变小
了。这时使用如下命令，就能查找到仍然未关闭的文件描述符，以及对应的进程 id:

```shell
lsof | grep delete
```

解决方法：

- 最简单有效的方法，就是关闭或者重启正在占用文件的进程。但是可能会造成中断，需要在搞清楚影响后再确认
  何时进行操作。
- 对于日志文件，在紧急情况下也有更简单的方法：`echo "" > /proc/<pid>/fd/<fd>` 直接通过 `/proc` 清空
  仍然被占用的文件。之后再另找时间重启这个进程。

### 插入硬盘后未识别到分区

现象：插入硬盘后使用 `lsblk` 查看只能看到硬盘，无法看到对应的分区，也就无法挂载分区。

以 `/dev/sda` 硬盘为例，排查流程如下：

- 切换到 root 用户
- 使用 `fdisk -l /dev/sda` 查看硬盘，报错
  `The primary GPT table is corrupt, but the backup appears OK, so that will be used.`，并且正确地列
  出了磁盘分区

这样就能确认是分区表损坏导致的问题，解决方法（参考
[Fixing corrupt backup GPT table? - Stack Exchange](https://askubuntu.com/questions/386752/fixing-corrupt-backup-gpt-table)）：

```shell
# 使用 fdisk 修复分区表
> fdisk /dev/sda
Welcome to fdisk (util-linux 2.37.4).
Changes will remain in memory only, until you decide to write them.
Be careful before using the write command.

The primary GPT table is corrupt, but the backup appears OK, so that will be used.

# 首先敲 p 确认分区正确性
Command (m for help): p

Disk /dev/sda: 3.64 TiB, 4000787030016 bytes, 7814037168 sectors
Disk model: EJRX-89T1XY0
Units: sectors of 1 * 512 = 512 bytes
Sector size (logical/physical): 512 bytes / 4096 bytes
I/O size (minimum/optimal): 4096 bytes / 4096 bytes
Disklabel type: gpt
Disk identifier: 9E1C5702-ECBB-45EC-9191-7BE8691D656E

Device     Start        End    Sectors  Size Type
/dev/sda1   2048 7814035455 7814033408  3.6T Microsoft basic data

# 然后直接敲 w，写入分区表，就能修复问题
Command (m for help): w
The partition table has been altered.
Calling ioctl() to re-read partition table.
Syncing disks.
```
