# 性能问题排查路径

> 如下内容中涉及网络的部分，均不适合在公网使用！本文假设整个网络都是安全可信的环境，因此均使用了无保
> 护的 HTTP 监听。这要求整个环境是一个局域网，终端电脑也在局域网内，或者通过 socks/http 代理、VPN 等
> 手段加入了该局域网。

1. 常规地检查 cpu/mem 利用率、网络 IO 与磁盘 IO、TCP 连接情况，看看有无异常
   1. 可以使用常规的各种命令行工具（麻烦），也可以使用
      [prometheus/node_exporter](https://github.com/prometheus/node_exporter) 配合
      [Node Exporter Full](https://grafana.com/grafana/dashboards/1860-node-exporter-full/) 查看机器
      的各种详细指标（方便快捷）。
2. 如果都没发现问题，对于 JVM 应用，可以直接使用 [arthas](https://github.com/alibaba/arthas) 绘制火
   焰图、捕获
   [java-flight-recorder-monitoring](https://www.baeldung.com/java-flight-recorder-monitoring) 数
   据。
   1. 火焰图跟 JFR 文件通常需要在本地图形化界面才能进行分析，而 arthas 本身默认监听 127.0.0.1，我改
      成监听 0.0.0.0 发现报错。
   2. 解决方法：使用 socat 多做一层转发以启用 0.0.0.0 的监
      听：`socat TCP-LISTEN:5432,fork,reuseaddr  TCP:localhost:3658`
3. 对于 JVM/Go/NodeJS/Python 等服务，并且运行在 Kubernetes 上，可以使用
   [kubectl-flame](https://github.com/yahoo/kubectl-flame) 画出 CPU 火焰图，定位最耗时的操作。
   1. 可以直接使用一行 Python 命令启动一个文件服务器，实现直接通过浏览器浏览并查看 svg 火焰图。
   2. `python3 -m http.server 9000 --bind 0.0.0.0 --directory /tmp/`，详见
      [http.server — HTTP servers](https://docs.python.org/3/library/http.server.html#http.server.SimpleHTTPRequestHandler.do_GET)
