## Linux 使用远程桌面

远程桌面在远程 Linux 主机开发调试时是比较需要的功能，查了一圈最成熟稳定的方案貌似有两个：

- SSH X11 Forwarding: 这是最简单的方案，在局域网下，而且客户端就是 Linux 时体验非常好，很适合用于偶
  尔跑些图形任务。
- [xrdp](https://github.com/neutrinolabs/xrdp): 它是 Windows 远程桌面协议的 Linux 实现，好处是性能不
  错，而且你还可以使用全平台的
  [Microsoft Remote Desktop](https://learn.microsoft.com/en-us/windows-server/remote/remote-desktop-services/clients/remote-desktop-clients)
  连接你的 Linux 远程桌面，能够自动做剪切板同步、音频转发、使用客户端 GPU 进行图形渲染加速等，还支持
  多屏操作，体验很好。
- VNC: VNC 是 Linux 下非常流行的远程桌面协议，它实现简单但不够安全，而且不支持传输声音，一般仅用于偶
  尔登录下服务器使用。

经我测试发现，局域网环境下，SSH X11 Forwarding 是最好用的，不需要任何额外配置，直接
`ssh -X user@host` 就能用了！而且仅在运行 GUI 应用时才会开启 GUI 窗口，与使用本机 APP 的体验别无二
致。

相比之下 xrdp xrdp 我第一次跑时就黑屏了，一番查找才解决，还是有一定的工作量才能把环境配好的，而且
xrdp 需要在服务端安装完整的桌面环境，要占据服务端更多的存储，渲染时估计也会更吃资源。

> 总结：Linux 推荐首选 SSH X11 Forwarding，而如果你的客户端是 Windows 或者 MacOS，或者你的 Linux 服
> 务器上已经安装了完整的桌面环境，那么 xrdp 是个不错的选择。

### 1. SSH X11 Forwarding

SSH X11 Forwarding 能把 client 当成一台显示器来用，尤其是在客户端就是 Linux 桌面的情况下，局域网使用
体验很好。不过它没做啥优化，数据都是全量发送，建议仅在局域网使用.

另外 SSH X11 Forwarding 不需要完整的桌面环境，不需要在 Linux 远程主机上装完整的 xfce/gnome 等环境，
这也是一个很大的优势。

先说下前置条件：

- 如果客户端桌面也是 Linux，那不需要改任何东西，直接就能用，体验也是三个桌面中最佳的。
- 如果客户端是 MacOS，则需要提前安装好 [XQuartz](https://www.xquartz.org/)
- 如果客户端是 Windows，则需要先安装好 [xming](https://sourceforge.net/projects/xming/)，可能还需要
  其他配置，我没试过请自行搜索解决。

然后首先修改 Linux 服务器的 ssh 配置 `/etc/ssh/sshd_config`，启用 `X11Forwarding` 功能：

```shell
# 启用 X11Forwarding
echo 'X11Forwarding yes' | sudo tee /etc/ssh/sshd_config
sudo systemctl restart ssh
```

最简单的用法就是 ssh 直接加个 `-X` 命令：

```shell
ssh -X user@host
```

加了 `-X` 后的命令会等待一会儿才进入远程终端，然后就可以直接运行 X11 应用了。

[How to start a GUI software on a remote Linux PC via SSH](https://askubuntu.com/questions/47642/how-to-start-a-gui-software-on-a-remote-linux-pc-via-ssh)
中介绍了多种用法，

> 如果你想远程使用更新的 wayland 系统，可以看看
> [waypipe](https://gitlab.freedesktop.org/mstoeckl/waypipe)。

### 2. 安装使用 [xrdp](https://github.com/neutrinolabs/xrdp)

xrdp 远程连接得到的是一个完整的图形化桌面，因此需要先在远程主机上安装好桌面环境，服务器场景下建议使
用性能最佳的 Xfce（或者直接用窗口管理器 i3wm），安装方式如下：

```shell
sudo apt update
sudo apt install xfce4 xfce4-goodies xorg dbus-x11 x11-xserver-utils
```

安装完成后最好重启下系统。

```shell
# 安装 xrdp 与它底层使用的 xorgxrdp
sudo apt install xrdp xorgxrdp

# Xrdp uses the /etc/ssl/private/ssl-cert-snakeoil.key, give it the permission to access it
sudo adduser xrdp ssl-cert

sudo systemctl enable xrdp
sudo systemctl restart xrdp
sudo systemctl status xrdp
```

另外还需要注意下 Linux 的防火墙配置，开放 3389 端口。

然后就能直接在客户机上连接远程主机的桌面了：

1. Windows 可直接使用自带的 `remote` 程序进行连接
2. MacOS/Android/IOS 系统，可使用跨平台且体验更佳的
   [Microsoft Remote Desktop](https://learn.microsoft.com/en-us/windows-server/remote/remote-desktop-services/clients/remote-desktop-clients)
   来连接远程主机。
3. 对于 Linux，可使用 GUI 程序 [Remmina](https://wiki.archlinux.org/title/Remmina) 进行连接，安装命
   令为 `yay -S remmina remmina-plugin-rdesktop freerdp`
   1. 连接时 Protocol 要选择 rdp，不能选 rdesktop，否则无法连接。
   2. 内网可将色深、画质调到最高，并启用文件夹共享、音频在 Local 播放，体验最佳（非最高画质不会显示
      桌面背景图）。
   3. 注意 freerdp 也必须安装，否则会无法连接。

如果连接完是黑屏，可以先等待一会儿，说不定就 OK 了...如果超过 10s 一直黑屏，可以使用如下命令改下
xrdp 的启动环境变量，再重新连接试试：

```shell
cat <<EOF | sudo tee /etc/xrdp/startwm.sh
unset DBUS_SESSION_BUS_ADDRESS
unset XDG_RUNTIME_DIR
EOF

sudo systemctl restart xrdp
```

要是解决不了，可以看看 Arch Linux WIKI 中的常见问题及解法
[Xrdp - Arch Linux WIKI](https://wiki.archlinux.org/title/xrdp)
