# 第二层：数据链路层（The Link Layer）

运行链路层协议的任何设备都可称为节点(Node)，比如主机、交换机、路由器、WiFi接入点(AP)等。而连接相邻节
点的通信信道就称为**链路(Link)**。

**数据链路层的协议定义了在单个链路上如何传输数据帧，以及简单的错误检测与纠正**。

链路层介于物理层和网络层之间，是一个承上启下的协议层。「链路层」与物理媒介是强关联的，而链路层以上的
协议（从网络层开始），就与物理层完全解耦了，从而实现跨媒介通讯。

## 各链路层协议的相同点与不同点

各链路层协议的数据帧结构，很可能也不一样。比如 Bluetooth、WiFi、Ethernet 这三个协议的链路层帧结构，
就都是不一样的。

但是为了实现一个统一可互联的网络，它们也存在很多相同点。

最大的一个相同点就是 MAC 地址，IPv4 使用的 ARP 地址解析协议，以及 IPv6 使用的 NDP 邻居发现协议，都是
以 MAC 地址为基础的。

> 但是注意 Bluetooth 就没有 MAC 地址，蓝牙 4.2 之前也不支持 ipv4/ipv6 协议。Bluetooth 蓝牙协议，是一
> 个横跨「物理层」到「应用层」的协议，

## 链路层寻址与 ARP 协议

为了使网络体系结构中各层次成为极为独立的构建模块，方便灵活地更换各层的实现，不同的层次需要有它们自己
的寻址方案。比如：

1. 应用层：主机名（DNS 域名或者 IP 地址）
2. 网络层：IP 地址
3. 链路层：MAC 地址

我们已经知道 DNS 协议是用于将一个域名解析为一个 IP 地址，这里的 ARP 协议很类似，它将一个 IP 地址解析
为一个 MAC 地址。而且为了加快解析速度，DNS 协议使用了缓存(TTL)。ARP 也一样有 ARP 缓存和过期时间
TTL。DNS 有缓存污染，ARP也同样有缓存投毒攻击。

## 局域网与广域网

### 1. 局域网（Local Network, LAN）

##### 以太网（Ethernet）与接口（Interface）名称

我们最常接触的都是网络层以上的协议，而以太网则是一个局域网的数据链路层协议。

以太网是目前使用最广泛的有线局域网技术。我们自己买的各种有线路由器，它的端口名称基本都是
`eth0`/`eth1` 等，这就是 `ethernet x` 的缩写。

### 2. 广域网接入技术

以太网是有线局域网技术，那显然广域网的数据链路层用的会是别的技术。

由于以太网没有访问控制、计费这两项广域网(ISP 网络服务提供商)需要的功能，而 PPP 协议(Point-to-Point
Protocol) 填补了这个欠缺，目前局域网接入广域网网最常用的是 PPPoE(PPP over Ethernet) 协议。

## 链路层加密技术

以太网协议以网线为传输介质，没有考虑过窃听，也就是说它是不加密的。数据只在网线内传输，这样做问题不
大。

但是对于无线局域网WiFi，或者移动网络2G/3G/4G/5G 而言，就不一样了，数据会被直接通过无线电广播出来。如
果不加密，那就相当于在外裸奔。

因此我们的移动电话网络，和 WiFi 网络，都是在链路层就进行了加密的。

需要注意的是，**WiFi/移动网络的链路层加密，针对的只是无线电这一段链路**。你的数据最终还是会进入到路
由器/移动信号塔，然后被转换成光信号/电信号进行传输，然后最终可能又通过以太网协议进入到某台服务器内。

## 参考

- [以太网与互联网有什么区别？](https://www.zhihu.com/question/19918493)
- [为什么我的网卡不叫eth0，而叫ens33、ens160、eno1 or enp0s\*？](https://blog.csdn.net/bigbaojian/article/details/86714248)
- [TCP/IP哪些层加密了哪些没有加密？](https://www.zhihu.com/question/52105092)
- [ARP 协议解析 - cizixs](https://cizixs.com/2017/07/31/arp-protocol/)
