# 统一的账号权限系统

当一个组织或企业的用户和PC越来越多时，这么多用户的分组与权限管理，谁能使用哪台电脑？能访问哪些服务？
有没有修改的权限？这些都会成为一个复杂的问题。

企业的中心化账号权限系统，就是解决这个问题的方案。

## 开源的账号权限管理系统

- [keycloak](https://github.com/keycloak/keycloak): 这是一个 13 年开发的开源系统，提供良好的 Web
  UI，开箱即用，支持 OIDC OAuth2 等多种协议。
- [ory/kratos](https://github.com/ory/kratos): 新一代的，云原生账号权限管理系统，支持 MFA 多因素认
  证、FIDO2 硬件认证、账号注册与恢复、第三方账号登录等等

以及一系列支持 OIDC 和 OAuth2.0 的开源 Provider，它们本身不提供身份管理的功能:

- [hydra](https://github.com/ory/hydra)
- [dex](https://github.com/dexidp/dex)

## Windows Active Directory

> 这是一个比较传统的方案，不过目前来看我们更愿意选择开源的方案，毕竟单独维护一台 Windows 服务器做域
> 控，感觉完全没必要啊。而且要保证高可用，还有比较高的学习成本，这套技术又和主流的开源技术完全脱轨。

AD 域，是微软提供的一个网络身份认证(identity)与网络资源访问管理(access management)服务。它被广泛应用
在各企业/组织的统一身份认证与资产管理中，主要用途有：

1. 用户相关：
   1. 集成验证(LDAP+Kerberos): 使用同一个身份登录所有企业应用、企业设备。
      1. 如 Wiki(Confluence)/Windows系统/Gitlab/Jenkins/云平台
   2. 单点登录(OSS): 如果支持 OSS，那用户只需要登录一次，就能通过该身份访问所有企业内部的应用。
   3. 对用户进行统一的分组、权限控制。
2. 资产管理：分组管理办公电脑、打印机等。
   1. 使用域控能在域 PC 上自动执行脚本，管理员拥有 PC 的最高权限！

它在 Linux 上的开源替代品是 LDAP+Kerberos，这两个协议都有相关的开源实现。

> 如果你不想使用 Windows 服务器，可以考虑直接使用群晖提供的 LDAP Server 或者 Directory Server，或者
> 自己使用开源项目搭建。

## Windows AD 域的建立与管理

要想让企业内部成员方便地使用各类内部服务，最好的办法就是组建一个 AD 域/LDAP 域控，统一使用同一套身份
验证系统登录使用各类应用，如 Gitlab、Wiki、NAS、CI/CD、Docker 镜像仓库、APP 包仓库等等。通过集成
radius 认证，还可以直接使用域控账号连接公司的 WiFi.

AD 域其实就是一个单纯的目录系统，或者叫文件夹系统。通过 UI 界面在 AD 域中创建群组/用户，和通过「文件
资源管理器」在文件系统中创建「文件夹」/「文件」非常类似。

参考：

- [AD域控 - 盐树枝](https://zhuanlan.zhihu.com/p/102694636)

## 最佳实践

我目前想到的 AD 域最佳配置：

1. 管理员账户 administrator 设置强密码，保存好密码，然后封存。紧急情况才使用这个账户。
2. 新建一个 ldap-ops 账户，给它加入域控的权限，专门用于各类应用、主机加入域控使用。
3. 将办公 PC 加入域控后，需要将该 PC 用户的域账号加入到【本地用户和组】-【本地管理员组】中！
   1. 【本地管理员组】只拥有对该 PC 的管理权限，域控仍然是安全的。
   2. 如果不将用户加入【本地管理员组】，很多对 C盘/注册表的操作（比如安装软件）都会要求通过管理员账
      户授权（输入账号密码），导致管理员账户密码半公开，域控安全形同虚设。
   3. 当然如果你们的办公 PC 要求更高的安全性，就是不允许域用户安装额外的软件，那就不需要这样做。
4. CI/CD 自动化使用专用账户访问域内各项服务（Gitlab/FTP服务器等）。
   1. 细分下去，还可以分为 frontend-ops、backend-ops、autotest-ops、operation-ops 等 CI/CD 账户。

## 进阶

- [Windows认证 | 域认证 - Kerberos 协议](https://zhuanlan.zhihu.com/p/89399579)

## 参考

- [Active Directory Domain Service 入门 - 官方文档](https://docs.microsoft.com/zh-cn/windows-server/identity/ad-ds/ad-ds-getting-started)
- [Active Directory的基本概念](https://www.cnblogs.com/IFire47/p/6672176.html)
- [LDAP, Active Directory and Federated Identity: What You Need to Know](https://doubleoctopus.com/blog/ldap-active-directory-and-federated-identity-what-you-need-to-know/)
