# 字符串

## Rune 类型：Code Point

Rune 代表 uncode 字符集中的一个字符，它本身是 int32 类型的别名。

通过 range 对 string 进行迭代，得到的是 rune 和其索引。

```go
package main

import (
	"fmt"
	"strconv"
)

func main() {
	text = "春天到了，hhhh"
	// iter runs on a string
	for i, char := range text {
		fmt.Println(i, string(char), char)
	}

	// convert string to a rune list
	rune_slice := []rune(text)
	fmt.Println(rune_slice)
}
// =============== 输出 =============
// 0 '春' 26149
// 3 '天' 22825
// 6 '到' 21040
// 9 '了' 20102
// 12 '，' 65292
// 15 'h' 104
// 16 'h' 104
// 17 'h' 104
// 18 'h' 104
```

因为 rune 其实是一个 int32 类型，所以直接打印出来得到的将是一个数字，需要将它转换成一个字符串字面
量，才能输出对应的字符。

在迭代一个 string 时，如果遇到无法解码的 bytes，将会返回一个特殊的 rune `\uFFFD`，打印出来是「�」.
因此如果遇到这个特殊 rune，通常意味着 string 的字符编码有错误。

## strings 包

一个字符串处理的实用工具包，工具函数有：

1. `Compare`: 字符串比较
2. `Contains`: 字符串是否包含其他 sub_str, runes, chars
3. `Index`: 查找索引
4. `Join`: 字符串合并
5. `Split`: 字符串切分
6. `Trim`: 掐头去尾
7. `Replace`: 字符串替换
8. `ToLower`/`ToUpper`/`ToTitle`: 大小写转换
9. `Builder`: 字符串拼接

等等

## strconv 包

用于处理字符串和各种数据类型之间的转换，比如：

1. rune 和 string 之间的转换：`strconv.QuoteRune`
1. 在字符串(比如 `-20`)与对应的 go 类型(`-20`) 之间互转。
1. 其他各种字符串相关的转换操作

字符串与数字之间的转换：

```golang
// integer to ASCII
a := strconv.Itoa(123) // => "123"
b := fmt.Sprintf("%d", 123)  // => "123"

// ASCII to integer
i, err := strconv.Atoi("123") // => 123
j, err := strconv.ParseInt("123", 10, 64)  // base 10, up to 64bits
```

## unicode 包

提供了一些对 rune 的判断方法：IsDigit IsLetter IsUpper IsLower...

以及对应的转换方法：toUpper toLower...

strings 也提供完全类似的方法，只是操作对象是 string 而不是 rune.
