# 性能测试（Performance Testing）

性能压测是通过自动化的测试工具模拟多种正常、峰值以及异常负载条件来对系统的各项性能指标进行测试。

## 性能测试的用处

1. 新系统上线支持在新系统上线前，通过执行性能压测能够对系统的负载能力有较为清晰的认知，从而结合预估
   的潜在用户数量保障系统上线后的用户体验。
2. 技术升级验证在系统重构过程中，通过性能压测验证对比，可以有效验证新技术的高效性，指导系统重构。
3. 业务峰值稳定性保障在业务峰值到来前，通过充分的性能压测，确保大促活动等峰值业务稳定性，保障峰值业
   务不受损。
4. 站点容量规划通过性能压测实现对站点精细化的容量规划，指导分布式系统机器资源分配。预估费用
5. 性能瓶颈探测通过性能压测探测系统中的性能瓶颈点，进行针对性优化，从而提升系统性能。

## 性能测试的各种类型

从**测试目的**上性能测试又可以划分为负载测试、压力测试、并发测试、配置测试以及可靠性测试。虽然也可以
从别的方面对性能测试进行分类，不过我们主要就只关注这四类：

1. 负载测试(Load Testing)：测试当负载逐渐增加时，系统各项性能指标的变化情况。
2. **压力测试(Stress Testing)**：通过确定一个系统的瓶颈或者不能接受的性能点，来获得系统能提供的最大
   服务级别的测试。
3. 并发测试(Concurrency Testing)：通过模拟用户并发访问，测试多用户并发访问同一个软件、同一个模块或者
   数据记录时是否存在死锁等性能问题。
4. 配置测试(Configuration Testing)：通过对被测系统的软/硬件环境的调整，了解各种不同配置对软件系统的
   性能影响的程度，从而找到系统各项资源的最优分配原则。
5. 可靠性测试(Reliability Testing)/稳定性测试(Stability Testing)：在给系统加载一定业务压力的情况下，
   使系统运行一段时间，以此检测系统是否稳定。

大家平常挂在嘴边最多的，是**压力测试**，前面讲过的「性能测试的用处」，主要依据都是压力测试的结果。

所有的性能测试，都是在测试一定条件下，对系统施加某种程度的压力，观察在此种压力下系统的状态。因此下面
将性能测试称为**性能压测**。

总的来说，性能压测是在对系统性能有一定程度了解的前提下，在确定的环境下针对压测需求进行的一种测试。

也有从零开始的系统首次进行压测，以全面了解新系统当前的性能（通常都会很差hhh）。

## 性能测试的难点

性能测试最大的难点，在于它的模糊性。

和编程相比，它更类似生物化学物理的实验，总是存在各种干扰项，很多时候测出来的数据自己都不知道怎么解
释。这就需要对操作系统、计算机网络有比较深的理解，或者别人设计好的现成的测试流程。

## 性能测试的注意事项

性能测试不是说一定要将系统打压到极限！

通常在压测时，要注意不要让打压机的系统的负载过高！通常 cpu 利用率峰值不应超过 30%-40%，而且在总体
cpu 利用率超过 5% 时，usr cpu 的利用率应该总是大于 sys cpu 利用率。

这样压测出的结果才不会受系统性能影响，否则压测出的曲线可能会出现奇怪的波动，这很可能就是因为系统拖了
后腿。

## 从零开始进行性能测试

一次完整的性能测试流程如下：

1. **确定性能压测目标**：性能压测目标可能源于项目计划、业务方需求等
2. **确定性能压测环境**：为了尽可能发挥性能压测作用，性能压测环境应当尽可能同线上环境一致
3. **确定性能压测通过标准**：针对性能压测目标以及选取的性能压测环境，制定性能压测通过标准，对于不同
   于线上环境的性能压测环境，通过标准也应当适度放宽
4. **设计性能压测**：编排压测链路，构造性能压测数据，尽可能模拟真实的请求链路以及请求负载
5. **执行性能压测**：借助性能压测工具，按照设计执行性能压测
   - 这一步除了性能压测工具本身的数据外，我们还需要关注应用的监控、日志、链路追踪等等各方面的数据！
6. **分析性能压测结果报告**：分析解读性能压测结果报告，判定性能压测是否达到预期目标，若不满足，要基
   于性能压测结果报告分析原因

由上述步骤可知，一次成功的性能压测涉及到多个环节，从场景设计到施压再到分析，缺一不可。工欲善其事，必
先利其器，而一款合适的性能工具意味着我们能够在尽可能短的时间内完成一次合理的性能压测，达到事半功倍的
效果。

## 名词解释：并发、RPS 和 RT

接触性能测试的同学要理解的概念有非常多，在正文之前先跟大家就几个核心指标统一下口径：

1. 并发用户、并发、VU：一般用来表示虚拟用户（Virtual User，简称VU），对应到 Jmeter 的线程组线程，对
   应到 Loadrunner 的并发 Concurrency ，在本文都是一个意思。
   - 并发用户数：指同时请求系统的用户个数，或者说同一时刻，系统正在处理的请求个数。
   - 在 Locust 中，它大致对应 `wait_time = constant(0)` 的情况下（请求之间不做任何等待，以保证系统的
     并发数不会出现大波动），users 的个数。
2. **RPS(Requests Per Second)**：每秒请求数。
   - **吞吐量(Throughput)**：指系统在单位时间内处理请求的数量。和这里的 RPS 基本可以等同。
   - QPS(Query Per Second): 每秒查询数：仅针对查询请求(GET)的 RPS
   - 每秒事务数 TPS(Transaction Per Second)：不单独讨论“事务”时，可以将它等同为这里的 RPS
3. **响应时间、RT**：对，没错，这个就是你理解的那个意思，从发起请求到完全接收到应答的时间消耗。
   - 主要关注响应时间的 95 线和 90 线。

上述几个参数中，RPS 和 RT，是两个具有重要指导意义的参数，最值得关注。

而并发用户数，在不引入任何等待时间的前提下，基本没有什么指导意义，一般不需要理会。因为真实环境下的并
发用户们，每次操作之间都会有随机的等待时间。不做任何等待的情况下测到的并发用户数，完全体现不了系统可
服务的真实用户量。

## 压测链路

在产品开发的过程中，压测链路的设计应该也是一个动态的过程。

在产品初期，我们可以根本没有所谓的链路，而只是对每个 API 单独进行压测——这或许可以称为单接口压测。因
为在产品初期，简单的单接口压测就已经能测出很多的问题了。而且为了简化问题，最好是能排除掉其他接口对当
前接口的影响。

单接口压测没问题了后，才会考虑进行链路压测。

### 通过压测进行系统瓶颈分析

1. 手动进行压测，压测出最优 RPS、RT
2. 分析被压测应用的服务器监控数据、判断是否是服务器性能瓶颈
   - 关注数据：Load1/Load5/CPU/RAM/磁盘IO/网络IO/TCP连接数
3. 查看链路追踪信息，确认是否是 RPC/Redis/MySQL 等远程调用的瓶颈。
   1. 也可以考虑接入阿里云 DAS 等数据库性能分析服务，方便定位问题。
   2. SQL 数据瓶颈的解决方法：加索引 => 优化 SQl => 分库分表
   3. 也可以借助 MySQL 的 `EXPLAIN` 进行 SQL 语句分析。
4. 可能是应用本身的性能问题，查看应用日志与源码。
5. 如果还是没找到瓶颈，考虑是否是服务器/网络/容器集群配置不当。

## 性能测试工具

- 可编程的性能测试工具
  - [Locust](https://github.com/locustio/locust): 使用 python 编写测试脚本，非常适合编写复杂的链路压
    测。
  - [grafana/k6](https://github.com/grafana/k6): grafana 开源的性能测试工具，很受欢迎，生态也很好。
    仅支持使用 js 编写测试脚本。而且官方有提供工具把 har/jmeter/postman 文件翻译成 k6 脚本。
  - [wrk](https://github.com/wg/wrk): 压力测试的不二之选，能够使用单机生成非常大的压力，而且还支持使
    用 lua 编写压测脚本。
  - [fortio](https://github.com/fortio/fortio): Istio 的御用负载测试工具
- 其他工具：
  - [JMeter](https://github.com/apache/jmeter): 貌似是 java 界最流行的性能测试工具
  - [ab](https://httpd.apache.org/docs/2.4/programs/ab.html): apache server 附带的性能测试工具，历史
    悠久。但是它是单线程的，没有 wrk 那么强悍，也就不适合用于压力测试。
  - [siege](https://github.com/JoeDog/siege): 又一款 c 语言实现的压测工具，暂时不清楚和 wrk 有多大区
    别
  - [vegeta](https://github.com/tsenart/vegeta): 这玩意在 github 上 star 非常高，go 写的，但是用起来
    感觉数据不太对劲
    - 2020/10 开始就没更新了
  - [hey](https://github.com/rakyll/hey): 又一款 go 语言实现的压测工具，自称是 ab 的替代品，使用也和
    ab 一样简单。k9s 就集成了它。但是它的准确性也有待考证。
    - 2021/04 开始就没更新了

目前感觉这一串列表里，locust 跟 grafana/k6 是最好用的，这两个里面 k6 是最漂亮、精致的。

## 参考

- [阿里云 - 性能测试技术指南](https://help.aliyun.com/document_detail/29337.html)
- [从压测工具谈并发、压力、吞吐量](https://zhuanlan.zhihu.com/p/23211458)
- [饿了么全链路压测的探索与实践](https://zhuanlan.zhihu.com/p/30306892)
- [浅谈服务器性能测试的全生命周期——从测试、结果分析到优化策略 ](https://wetest.qq.com/lab/view/?id=102)
- [并发模式与 RPS 模式之争，性能压测领域的星球大战](https://yq.aliyun.com/articles/709950?spm=5176.7946858.1219570.2.c8802542NeDh2Z)
- [性能压测工具选型对比 - 阿里巴巴中间件](https://mp.weixin.qq.com/s?__biz=MzU4NzU0MDIzOQ==&mid=2247486886&idx=2&sn=cd9f4b50afd3d6a03dfeb724905ee428)
